#ifdef _WIN32
#pragma warning( disable : 4996 )
#endif

#include <iostream>
#include "grabber.h"
#include "tmkernel.h"
#include "tinyxml.h"

namespace {
int GetValue(const TiXmlHandle &hRoot, const char *val_name) {
	TiXmlElement *pElem = hRoot.FirstChild(val_name).Element();
	if(!pElem)
		return 0;
	const char *val = pElem->GetText();
	return atoi(val);
}

void ParseAffixment(const char *affix_file_name, S_TMKER_Affixment &affix) {
 	TiXmlDocument doc(affix_file_name);
	if(!doc.LoadFile())
		throw "Failed to load xml file";

	TiXmlHandle hDoc(&doc);

	TiXmlElement *pElem = hDoc.FirstChildElement().Element();
	TiXmlHandle hRoot = TiXmlHandle(pElem);

	int version = GetValue(hRoot, "VERSION");
	if(version != 1)
		throw "Invalid affixment version";

	affix.nLanes = GetValue(hRoot, "LANES");
	char val_path[64];
	for(int i=0; i<affix.nLanes; i++) {
		sprintf(val_path, "DIRECTION_%d", i + 1);
		affix.nDirection[i] = GetValue(hRoot, val_path);

		sprintf(val_path, "LANE_WIDTH_%d", i + 1);
		affix.nLanesWidth[i] = GetValue(hRoot, val_path);
	}

	for(int i=0; i<4; i++) {
		sprintf(val_path, "EARTH_X%d", i + 1);
		affix.pptEarth[i].shX = static_cast<short>(GetValue(hRoot, val_path));

		sprintf(val_path, "EARTH_Y%d", i + 1);
		affix.pptEarth[i].shY = static_cast<short>(GetValue(hRoot, val_path));

		sprintf(val_path, "IMAGE_X%d", i + 1);
		affix.pptImage[i].shX = static_cast<short>(GetValue(hRoot, val_path));

		sprintf(val_path, "IMAGE_Y%d", i + 1);
		affix.pptImage[i].shY = static_cast<short>(GetValue(hRoot, val_path));

		if(i < 2) {
			sprintf(val_path, "LEFT_ROAD_EDGE_X%d", i + 1);
			affix.pptLeftRoadEdge[i].shX = static_cast<short>(GetValue(hRoot, val_path));

			sprintf(val_path, "LEFT_ROAD_EDGE_Y%d", i + 1);
			affix.pptLeftRoadEdge[i].shY = static_cast<short>(GetValue(hRoot, val_path));

			sprintf(val_path, "RIGHT_ROAD_EDGE_X%d", i + 1);
			affix.pptRightRoadEdge[i].shX = static_cast<short>(GetValue(hRoot, val_path));

			sprintf(val_path, "RIGHT_ROAD_EDGE_Y%d", i + 1);
			affix.pptRightRoadEdge[i].shY = static_cast<short>(GetValue(hRoot, val_path));
		}
	}
}

const int FRAMES = 1000;

}

int main(int argc, char **argv) {
	TMKER_DESC tmk = NULL;
	unsigned char *frame_data = NULL;

	try {
		 if(argc != 4)
			 throw "Use: example FRAMES_DIRECTORY AFFIXMENT_FILE_NAME FRAME_RATE";

		const char *frames_dir_name = argv[1];	
		const char *affix_file_name = argv[2];
		int frame_rate = atoi(argv[3]);

		C_Grabber grabber;
		grabber.Init(frames_dir_name);
		
		S_TMKER_Init tmk_init;
		tmk_init.nFrameWidth = grabber.FrameWidth();
		tmk_init.nFrameHeight = grabber.FrameHeight();
		tmk_init.nFrameRate = frame_rate;
		ParseAffixment(affix_file_name, tmk_init.Affix);

		if(TMKER_RES_OK != TMKER_Create(&tmk))
			throw "Error in TMKER_Create()";	

		TMKER_RESULT tmk_res = TMKER_Init(tmk, &tmk_init);

		if(TMKER_RES_DONGLE == tmk_res)
			std::cout << "***** DEMO MODE. *****" << std::endl;
		else if(TMKER_RES_OK != tmk_res)
			throw "TMKernel initialization error";
		
		frame_data = new unsigned char[tmk_init.nFrameWidth * tmk_init.nFrameHeight];
		if(!frame_data)
			throw "Memory allocation error";

		S_TMKER_Event tmk_event;

		for(int i=0; i<FRAMES; i++) {
			grabber.GetFrame(frame_data);
			TMKER_ProcessFrame(tmk, frame_data, &tmk_event);

			for(int i=0; i<tmk_init.Affix.nLanes; i++)
				if(tmk_event.Events[i].nVehicleLength)
					std::cout << "Lane: " << (i + 1) << ", Class: " << tmk_event.Events[i].nVehicleClass << 
							", Speed: " << tmk_event.Events[i].nVehicleSpeed << std::endl;
		}
	}
	catch(const char *err) {
		std::cout << err << std::endl;
	}

	TMKER_Destroy(tmk);
}


